"""
This module contains routines to set up and manage the metadata for a MAST-ML run

Mastml:
    Class to set up directories for saving the output of a MAST-ML run, and for constructing and updating a
    metadata summary file.

"""

import os
from datetime import datetime
from collections import OrderedDict
import json

class Mastml():
    """
    Main helper class to initialize mastml runs and create and manage run metadata

    Args:
        savepath: (str), string specifing the savepath name for the mastml run

        mastml_metdata: (dict), dict of mastml metadata. If none, a new dict will be created

    Methods:
        _initialize_run: initializes run by making new metadata file or updating existing one, and initializing the output directory.
            Args:
                None

            Returns:
                None

        _initialize_output: creates the output folder based on specified savepath and datetime information
            Args:
                None

            Returns:
                None

        _initialize_metadata: creates a new metadata file and saves the savepath info to it
            Args:
                None

            Returns:
                None

        _update_metadata: placeholder for updating the metadata file with new run information
            Args:
                None

            Returns:
                None

        _save_mastml_metadata: saves the metadata dict as a json file
            Args:
                None

            Returns:
                None

        get_savepath: returns the savepath
            Args:
                None

            Returns:
                string specifying the savepath of the mastml run

        get_mastml_metadata: returns the metadata file
            Args:
                None

            Returns:
                mastml metadata object (ordered dict)

    """
    def __init__(self, savepath, mastml_metadata=None):
        self.savepath = savepath
        self.mastml_metadata = mastml_metadata
        self._initialize_run()

    def _initialize_run(self):
        self._initialize_output()
        if self.mastml_metadata is None:
            self._initialize_metadata()
        #else:
        #    self._update_metadata()
        self._save_mastml_metadata()

    def _initialize_output(self):
        # Make an output folder for the run to store all data to
        if os.path.exists(self.savepath):
            try:
                os.rmdir(self.savepath)  # succeeds if empty
            except OSError:  # directory not empty
                print(f"{self.savepath} not empty. Renaming...")
                now = datetime.now()
                self.savepath = self.savepath.rstrip(os.sep)  # remove trailing slash
                self.savepath = f"{self.savepath}_{now.month:02d}_{now.day:02d}" \
                         f"_{now.hour:02d}_{now.minute:02d}_{now.second:02d}"
        os.makedirs(self.savepath)
        return

    def _initialize_metadata(self):
        self.mastml_metadata = OrderedDict()
        self.mastml_metadata['savepath'] = self.savepath
        return

    def _update_metadata(self,
                         outerdir,
                         split_name,
                         model=None,
                         splitter=None,
                         preprocessor=None,
                         selector=None,
                         hyperopt=None,
                         train_stats=None,
                         test_stats=None,
                         leaveout_stats=None,
                         X_train=None,
                         X_test=None,
                         X_leaveout=None,
                         X_extra_train=None,
                         X_extra_test=None,
                         X_extra_leaveout=None,
                         y_train=None,
                         y_test=None,
                         y_leaveout=None,
                         y_pred_train=None,
                         y_pred=None,
                         y_pred_leaveout=None,
                         residuals_train=None,
                         residuals_test=None,
                         residuals_leaveout=None,
                         model_errors_train=None,
                         model_errors_test=None,
                         model_errors_leaveout=None,
                         model_errors_train_cal=None,
                         model_errors_test_cal=None,
                         model_errors_leaveout_cal=None,
                         dataset_stdev=None):
        # Update with new entry: (1) module, (2) class, (3) path executed, (4) paths to data used ???
        if outerdir not in self.mastml_metadata.keys():
            self.mastml_metadata[outerdir] = OrderedDict()
        if split_name not in self.mastml_metadata[outerdir].keys():
            self.mastml_metadata[outerdir][split_name] = OrderedDict()
        if split_name == 'split_outer_dir':
            self.mastml_metadata[outerdir][split_name]['splitdir'] = outerdir
        else:
            self.mastml_metadata[outerdir][split_name]['splitdir'] = split_name
        if model is not None:
            try:
                model_name = model.model.__class__.__name__
            except:
                model_name = model.__class__.__name__
            self.mastml_metadata[outerdir][split_name]['model'] = model_name

            if split_name == 'split_summary':
                self.mastml_metadata[outerdir][split_name]['model_path'] = os.path.join(os.path.join(self.savepath, outerdir), model_name+'.pkl')
            elif split_name == 'split_outer_summary':
                self.mastml_metadata[outerdir][split_name]['model_path'] = os.path.join(outerdir, model_name+'.pkl')
            else:
                self.mastml_metadata[outerdir][split_name]['model_path'] = os.path.join(os.path.join(os.path.join(self.savepath, outerdir), split_name), model_name + '.pkl')
        if splitter is not None:
            self.mastml_metadata[outerdir][split_name]['splitter'] = splitter.splitter.__class__.__name__
        if preprocessor is not None:
            self.mastml_metadata[outerdir][split_name]['preprocessor'] = preprocessor.__class__.__name__
        if selector is not None:
            self.mastml_metadata[outerdir][split_name]['selector'] = selector.__class__.__name__
        if hyperopt is not None:
            self.mastml_metadata[outerdir][split_name]['hyperopt'] = hyperopt.__class__.__name__
        if train_stats is not None:
            self.mastml_metadata[outerdir][split_name]['train_stats'] = train_stats.to_json()
        if test_stats is not None:
            self.mastml_metadata[outerdir][split_name]['test_stats'] = test_stats.to_json() #to_dict
        if leaveout_stats is not None:
            self.mastml_metadata[outerdir][split_name]['leaveout_stats'] = leaveout_stats.to_json()
        if X_train is not None:
            self.mastml_metadata[outerdir][split_name]['train_columns'] = X_train.columns.tolist()
            self.mastml_metadata[outerdir][split_name]['X_train'] = X_train.to_json()
        if X_test is not None:
            self.mastml_metadata[outerdir][split_name]['X_test'] = X_test.to_json()
        if X_leaveout is not None:
            self.mastml_metadata[outerdir][split_name]['X_leaveout'] = X_leaveout.to_json()
        if X_extra_train is not None:
            self.mastml_metadata[outerdir][split_name]['X_extra_train'] = X_extra_train.to_json()
        if X_extra_test is not None:
            self.mastml_metadata[outerdir][split_name]['X_extra_test'] = X_extra_test.to_json()
        if X_extra_leaveout is not None:
            self.mastml_metadata[outerdir][split_name]['X_extra_leaveout'] = X_extra_leaveout.to_json()
        if y_train is not None:
            self.mastml_metadata[outerdir][split_name]['y_train'] = y_train.to_json()
        if y_test is not None:
            self.mastml_metadata[outerdir][split_name]['y_test'] = y_test.to_json()
        if y_leaveout is not None:
            self.mastml_metadata[outerdir][split_name]['y_leaveout'] = y_leaveout.to_json()
        if y_pred_train is not None:
            self.mastml_metadata[outerdir][split_name]['y_pred_train'] = y_pred_train.to_json()
        if y_pred is not None:
            self.mastml_metadata[outerdir][split_name]['y_pred'] = y_pred.to_json()
        if y_pred_leaveout is not None:
            self.mastml_metadata[outerdir][split_name]['y_pred_leaveout'] = y_pred_leaveout.to_json()
        if residuals_train is not None:
            self.mastml_metadata[outerdir][split_name]['residuals_train'] = residuals_train.to_json()
        if residuals_test is not None:
            self.mastml_metadata[outerdir][split_name]['residuals_test'] = residuals_test.to_json()
        if residuals_leaveout is not None:
            self.mastml_metadata[outerdir][split_name]['residuals_leaveout'] = residuals_leaveout.to_json()
        if model_errors_train is not None:
            self.mastml_metadata[outerdir][split_name]['model_errors_train'] = model_errors_train.to_json()
        if model_errors_test is not None:
            self.mastml_metadata[outerdir][split_name]['model_errors_test'] = model_errors_test.to_json()
        if model_errors_leaveout is not None:
            self.mastml_metadata[outerdir][split_name]['model_errors_leaveout'] = model_errors_leaveout.to_json()
        if model_errors_train_cal is not None:
            self.mastml_metadata[outerdir][split_name]['model_errors_train_cal'] = model_errors_train_cal.to_json()
        if model_errors_test_cal is not None:
            self.mastml_metadata[outerdir][split_name]['model_errors_test_cal'] = model_errors_test_cal.to_json()
        if model_errors_leaveout_cal is not None:
            self.mastml_metadata[outerdir][split_name]['model_errors_leaveout_cal'] = model_errors_leaveout_cal.to_json()
        if dataset_stdev is not None:
            self.mastml_metadata[outerdir][split_name]['dataset_stdev'] = dataset_stdev
        return

    def _save_mastml_metadata(self):
        with open(os.path.join(self.savepath, 'mastml_metadata.json'), 'w') as f:
            json.dump(self.mastml_metadata, f)
        return

    @property
    def get_savepath(self):
        return self.savepath

    @property
    def get_mastml_metadata(self):
        return self.mastml_metadata



import unittest
import numpy as np
import pandas as pd
import os
import sys
from mastml.datasets import LocalDatasets
sys.path.insert(0, os.path.abspath('../../../'))

from mastml.feature_selectors import NoSelect, EnsembleModelFeatureSelector, PearsonSelector, MASTMLFeatureSelector
from sklearn.ensemble import RandomForestRegressor

import mastml
mastml_path = mastml.__path__._path[0]

class TestSelectors(unittest.TestCase):

    def test_noselect(self):
        X = pd.DataFrame(np.random.uniform(low=0.0, high=100, size=(50, 10)))
        y = pd.Series(np.random.uniform(low=0.0, high=100, size=(50,)))
        selector = NoSelect()
        Xselect = selector.evaluate(X=X, y=y, savepath=os.getcwd())
        self.assertEqual(Xselect.shape, (50, 10))
        return

    def test_ensembleselector(self):
        X = pd.DataFrame(np.random.uniform(low=0.0, high=100, size=(50, 10)))
        y = pd.Series(np.random.uniform(low=0.0, high=100, size=(50,)))
        model = RandomForestRegressor()
        selector = EnsembleModelFeatureSelector(model=model, n_features_to_select=5)
        Xselect = selector.evaluate(X=X, y=y, savepath=os.getcwd())
        self.assertEqual(Xselect.shape, (50, 5))
        self.assertTrue(os.path.exists('EnsembleModelFeatureSelector_feature_importances.xlsx'))
        os.remove('EnsembleModelFeatureSelector_feature_importances.xlsx')
        os.remove('selected_features.txt')
        return

    def test_pearsonselector(self):
        X = pd.DataFrame(np.random.uniform(low=0.0, high=100, size=(10, 10)))
        y = pd.Series(np.random.uniform(low=0.0, high=100, size=(10,)))
        selector = PearsonSelector(threshold_between_features=0.3,
                                   threshold_with_target=0.3,
                                   flag_highly_correlated_features=True,
                                   n_features_to_select=3)
        selector.evaluate(X=X, y=y, savepath=os.getcwd())
        Xselect = selector.transform(X=X)
        self.assertEqual(Xselect.shape, (10, 3))
        self.assertTrue(os.path.exists('PearsonSelector_fullcorrelationmatrix.xlsx'))
        self.assertTrue(os.path.exists('PearsonSelector_highlycorrelatedfeatures.xlsx'))
        self.assertTrue(os.path.exists('PearsonSelector_highlycorrelatedfeaturesflagged.xlsx'))
        self.assertTrue(os.path.exists('PearsonSelector_highlycorrelatedwithtarget.xlsx'))
        os.remove('PearsonSelector_fullcorrelationmatrix.xlsx')
        os.remove('PearsonSelector_highlycorrelatedfeatures.xlsx')
        os.remove('PearsonSelector_highlycorrelatedfeaturesflagged.xlsx')
        os.remove('PearsonSelector_highlycorrelatedwithtarget.xlsx')
        os.remove('selected_features.txt')
        return

    def test_mastmlselector(self):
        X = pd.DataFrame(np.random.uniform(low=0.0, high=100, size=(10, 10)))
        y = pd.Series(np.random.uniform(low=0.0, high=100, size=(10,)))
        model = RandomForestRegressor()
        selector = MASTMLFeatureSelector(model=model, n_features_to_select=2, cv=None, manually_selected_features=[1])
        selector.evaluate(X=X, y=y, savepath=os.getcwd())
        Xselect = selector.transform(X=X)
        self.assertEqual(Xselect.shape, (10, 2))
        self.assertTrue(os.path.exists(os.path.join(os.getcwd(), 'MASTMLFeatureSelector_featureselection_data.xlsx')))
        os.remove(os.path.join(os.getcwd(), 'MASTMLFeatureSelector_featureselection_data.xlsx'))
        os.remove('selected_features.txt')
        return

    def test_featureselector_with_random_score(self):
        target = 'E_regression.1'
        extra_columns = ['Material compositions 1', 'Material compositions 2', 'Hop activation barrier', 'E_regression']
        d = LocalDatasets(file_path=os.path.join(mastml_path, 'data/figshare_7418492/All_Model_Data.xlsx'),
                          target=target,
                          extra_columns=extra_columns,
                          group_column='Material compositions 1',
                          testdata_columns=None,
                          as_frame=True)
        data_dict = d.load_data()
        X = data_dict['X']
        y = data_dict['y']
        model = RandomForestRegressor()
        selector = EnsembleModelFeatureSelector(model=model, n_features_to_select=100, n_random_dummy= 100)
        Xselect = selector.evaluate(X=X, y=y, savepath=os.getcwd())
        self.assertEqual(Xselect.shape, (408, 100))
        self.assertTrue(os.path.exists('EnsembleModelFeatureSelector_feature_importances.xlsx'))
        os.remove('EnsembleModelFeatureSelector_feature_importances.xlsx')
        os.remove('selected_features.txt')
        return

    def test_featureselector_with_permutated_score(self):
        target = 'E_regression.1'
        extra_columns = ['Material compositions 1', 'Material compositions 2', 'Hop activation barrier', 'E_regression']
        d = LocalDatasets(file_path=os.path.join(mastml_path, 'data/figshare_7418492/All_Model_Data.xlsx'),
                          target=target,
                          extra_columns=extra_columns,
                          group_column='Material compositions 1',
                          testdata_columns=None,
                          as_frame=True)
        data_dict = d.load_data()
        X = data_dict['X']
        y = data_dict['y']
        model = RandomForestRegressor()
        selector = EnsembleModelFeatureSelector(model=model, n_features_to_select=100, n_random_dummy= 100, n_permuted_dummy = 200)
        Xselect = selector.evaluate(X=X, y=y, savepath=os.getcwd())
        self.assertEqual(Xselect.shape, (408, 100))
        self.assertTrue(os.path.exists('EnsembleModelFeatureSelector_feature_importances.xlsx'))
        os.remove('EnsembleModelFeatureSelector_feature_importances.xlsx')
        os.remove('selected_features.txt')
        return

if __name__=='__main__':
    unittest.main()


import unittest
import numpy as np
import pandas as pd
import os
import sys
sys.path.insert(0, os.path.abspath('../../../'))

from mastml.plots import Scatter, Histogram

class TestPlots(unittest.TestCase):

    def test_scatter(self):
        X = pd.Series(np.random.uniform(low=0.0, high=100, size=(50,)))
        y = pd.Series(np.random.uniform(low=0.0, high=100, size=(50,)))

        Scatter().plot_predicted_vs_true(y_true=X,
                                         y_pred=y,
                                         savepath=os.getcwd(),
                                         x_label='TEST_scatter',
                                         data_type='test',)
        self.assertTrue(os.path.exists('parity_plot_test.png'))
        os.remove('parity_plot_test.png')
        return

    def test_histogram(self):
        X = pd.Series(np.random.uniform(low=0.0, high=100, size=(50,)))

        Histogram().plot_histogram(df=X,
                                   savepath=os.getcwd(),
                                   file_name='TEST_hist',
                                   x_label='TEST_hist')
        self.assertTrue(os.path.exists('TEST_hist.png'))
        self.assertTrue(os.path.exists('TEST_hist.xlsx'))
        self.assertTrue(os.path.exists('TEST_hist_statistics.xlsx'))
        os.remove('TEST_hist.png')
        os.remove('TEST_hist.xlsx')
        os.remove('TEST_hist_statistics.xlsx')
        return

    def test_residual_histogram(self):
        X = pd.Series(np.random.uniform(low=0.0, high=100, size=(50,)))
        y = pd.Series(np.random.uniform(low=0.0, high=100, size=(50,)))

        Histogram().plot_residuals_histogram(y_true=X,
                                             y_pred=y,
                                             savepath=os.getcwd())
        self.assertTrue(os.path.exists('residual_histogram.png'))
        self.assertTrue(os.path.exists('residual_histogram.xlsx'))
        self.assertTrue(os.path.exists('residual_histogram_statistics.xlsx'))
        os.remove('residual_histogram.png')
        os.remove('residual_histogram.xlsx')
        os.remove('residual_histogram_statistics.xlsx')
        return


if __name__ == '__main__':
    unittest.main()



########################################################################
# This is the setup script for the MAterials Simulation Toolkit
#   machine-learning module (MAST-ML)
# Creator and Maintainer: UW-Madison MAST-ML Development Team
#
########################################################################

from __future__ import print_function
import sys
from setuptools import setup, find_packages

###Python version check
#print "Python version detected: %s" % sys.version_info
if sys.version_info[0] < 3:
    print('Python Version %d.%d.%d found' % (sys.version_info[0], sys.version_info[1], sys.version_info[2]))
    print('Python version >= 3 needed!')
    sys.exit(0)

#print("Python version 3.6.6 is REQUIRED")
#print("Check with `python --version`")

# One of the techniques from https://packaging.python.org/guides/single-sourcing-package-version/
verstr = "3.0.1"
try:
    verstr = open("VERSION", "rt").read().strip()
except EnvironmentError:
    pass # Okay, there is no version file.

setup(
    name="mastml",
    packages=['mastml', 'mastml.magpie', 'mastml.tests.unit_tests', 'mastml.data'],
    package_data={'mastml.magpie': ["*.*"], 'mastml.tests.unit_tests': ["*.*"], 'mastml.data': ["*.*"]},
    include_package_data = True,
    version=verstr,
    install_requires=[
        "citrination-client",
        "dlhub_sdk",
        "globus_nexus_client",
        "globus_sdk",
        "matminer",
        "matplotlib",
        "mdf_forge",
        "mdf-toolbox",
        "mlxtend",
        "numpy",
        "openpyxl",
        "pandas",
        "pymatgen==2021.3.9",
        "scikit-learn>=0.24.1",
        "scikit-learn-extra",
        "scikit-optimize",
        "scipy",
        "sphinx-automodapi",
        ],
    author="MAST Development Team, University of Wisconsin-Madison Computational Materials Group",
    author_email="ddmorgan@wisc.edu",
    url="https://github.com/uw-cmg/MAST-ML",
    license="MIT",
    description="MAterials Simulation Toolkit - Machine Learning",
    long_description="MAterials Simulation Toolkit for Machine Learning (MASTML)",
    keywords=["MAST","materials","simulation","MASTML","machine learning"],
)



import unittest
import numpy as np
import pandas as pd
import os
import sys
import shutil
sys.path.insert(0, os.path.abspath('../../../'))

from mastml.datasets import SklearnDatasets, LocalDatasets, FoundryDatasets, MatminerDatasets
import mastml
mastml_path = mastml.__path__._path[0]

class TestDatasets(unittest.TestCase):

    def test_sklearn(self):
        sklearndata = SklearnDatasets(return_X_y=True, as_frame=True)
        bostonX, bostony = sklearndata.load_boston()
        irisX, irisy = sklearndata.load_iris()
        digitsX, digitsy = sklearndata.load_digits()
        diabetesX, diabetesy = sklearndata.load_diabetes()
        breast_cancerX, breast_cancery = sklearndata.load_breast_cancer()
        wineX, winey = sklearndata.load_wine()
        linnerudX, linnerudy = sklearndata.load_linnerud()
        self.assertEqual(bostonX.shape, (506, 13))
        self.assertEqual(irisX.shape, (150, 4))
        self.assertEqual(digitsX.shape, (1797, 64))
        self.assertEqual(diabetesX.shape, (442, 10))
        self.assertEqual(breast_cancerX.shape, (569, 30))
        self.assertEqual(wineX.shape, (178, 13))
        self.assertEqual(linnerudX.shape, (20, 3))
        return

    '''
    def test_figshare(self):
        FigshareDatasets().download_data(article_id='7418492', savepath=os.getcwd())
        self.assertTrue(os.path.exists('figshare_7418492'))
        return
    '''

    def test_local(self):
        target = 'E_regression.1'
        extra_columns = ['E_regression', 'Material compositions 1', 'Material compositions 2', 'Hop activation barrier']
        file_path = os.path.join(mastml_path, 'data/figshare_7418492/All_Model_Data.xlsx')
        d = LocalDatasets(file_path=file_path,
                          target=target,
                          extra_columns=extra_columns,
                          as_frame=True)
        data_dict = d.load_data()
        X = data_dict['X']
        y = data_dict['y']
        self.assertEqual(X.shape, (408, 287))
        self.assertEqual(y.shape, (408,))
        return

    def test_matminer(self):
        matminerdata = MatminerDatasets()
        df = matminerdata.download_data(name='dielectric_constant', save_data=True)
        self.assertTrue(os.path.exists('dielectric_constant.xlsx'))
        self.assertTrue(os.path.exists('dielectric_constant.pickle'))
        self.assertTrue(df.shape, (1056, 16))
        os.remove('dielectric_constant.xlsx')
        os.remove('dielectric_constant.pickle')
        return

    def test_foundry(self):
        foundrydata = FoundryDatasets(no_local_server=False, anonymous=True, test=True)
        foundrydata.download_data(name='pub_57_wu_highthroughput', download=False)
        return

if __name__ == '__main__':
    unittest.main()



import unittest
import numpy as np
import pandas as pd
import os
import sys
import shutil
sys.path.insert(0, os.path.abspath('../../../'))

from mastml.preprocessing import SklearnPreprocessor, MeanStdevScaler

class TestPreprocessor(unittest.TestCase):

    def test_sklearnpreprocessor(self):
        # Make toy data of random numbers
        X =  pd.DataFrame(np.random.uniform(low=0.0, high=100, size=(10,5)))

        preprocessor = SklearnPreprocessor(preprocessor='StandardScaler', as_frame=True)
        X_scaled = preprocessor.evaluate(X=X, savepath=os.getcwd(), file_name='test')

        self.assertEqual(X_scaled.shape[0], 10)
        self.assertTrue(os.path.exists(os.path.join(preprocessor.savepath, 'data_preprocessed_test.xlsx')))
        self.assertTrue(os.path.exists(os.path.join(preprocessor.savepath, 'StandardScaler.pkl')))
        os.remove('data_preprocessed_test.xlsx')
        os.remove('StandardScaler.pkl')

        return

    def test_meanstdevscaler(self):
        # Make toy data of random numbers
        X =  pd.DataFrame(np.random.uniform(low=0.0, high=100, size=(10,5)))

        preprocessor = MeanStdevScaler(mean=0, stdev=2, as_frame=True)
        X_scaled = preprocessor.evaluate(X=X, savepath=os.getcwd(), file_name='test')

        self.assertEqual(X_scaled.shape[0], 10)
        self.assertAlmostEqual(X_scaled[0].std(), 2)
        self.assertTrue(os.path.exists(os.path.join(preprocessor.savepath, 'data_preprocessed_test.xlsx')))
        self.assertTrue(os.path.exists(os.path.join(preprocessor.savepath, 'MeanStdevScaler.pkl')))
        os.remove('data_preprocessed_test.xlsx')
        os.remove('MeanStdevScaler.pkl')

        return

if __name__=='__main__':
    unittest.main()


import unittest
import numpy as np
import pandas as pd
import os
import sys
sys.path.insert(0, os.path.abspath('../../../'))

from mastml.metrics import Metrics

class TestMetrics(unittest.TestCase):

    def test_metrics(self):
        y = pd.Series(np.random.uniform(low=0.0, high=100, size=(50,)))
        metrics = Metrics(metrics_list=list())
        all_metrics = metrics._metric_zoo()
        metrics_list = all_metrics.keys()
        stats_dict = Metrics(metrics_list=metrics_list).evaluate(y_true=y, y_pred=y)
        return

if __name__=='__main__':
    unittest.main()


import unittest
import numpy as np
import pandas as pd
import os
import sys
import shutil
sys.path.insert(0, os.path.abspath('../../../'))

from mastml.data_cleaning import DataCleaning

class TestDataCleaning(unittest.TestCase):

    def test_datacleaning(self):
        X = pd.DataFrame(np.random.uniform(low=0.0, high=100, size=(50, 10)))
        y_input = pd.Series(np.random.uniform(low=0.0, high=100, size=(50,)))
        y_input.name = 'target'
        X_input = X.mask(np.random.random(X.shape) < 0.1)
        DataCleaning().remove(X=X_input, y=y_input, axis=1)
        DataCleaning().imputation(X=X_input, y=y_input, strategy='mean')
        DataCleaning().ppca(X=X_input, y=y_input)
        cleaner = DataCleaning()
        cleaner.evaluate(X=X_input, y=y_input, method='remove', axis=1, savepath=os.getcwd())
        self.assertTrue(os.path.exists(cleaner.splitdir))
        shutil.rmtree(cleaner.splitdir)
        return

if __name__=='__main__':
    unittest.main()


import unittest
import pandas as pd
import numpy as np
import os
import shutil
import sys
import sklearn.datasets as sk

sys.path.insert(0, os.path.abspath('../../../'))

from mastml.models import SklearnModel
from mastml.data_splitters import NoSplit, SklearnDataSplitter, LeaveCloseCompositionsOut, LeaveOutPercent, \
    Bootstrap, JustEachGroup, LeaveOutTwinCV, LeaveOutClusterCV

class TestSplitters(unittest.TestCase):

    def test_nosplit(self):
        X = pd.DataFrame(np.random.uniform(low=0.0, high=100, size=(10, 10)))
        y = pd.Series(np.random.uniform(low=0.0, high=100, size=(10,)))
        model = SklearnModel(model='LinearRegression')
        splitter = NoSplit()
        splitter.evaluate(X=X, y=y, models=[model], savepath=os.getcwd(), plots=list())
        for d in splitter.splitdirs:
            self.assertTrue(os.path.exists(d))
            shutil.rmtree(d)
        return

    def test_sklearnsplitter(self):
        X = pd.DataFrame(np.random.uniform(low=0.0, high=100, size=(10, 10)))
        y = pd.Series(np.random.uniform(low=0.0, high=100, size=(10,)))
        model = SklearnModel(model='LinearRegression')
        splitter = SklearnDataSplitter(splitter='KFold', shuffle=True, n_splits=5)
        splitter.evaluate(X=X, y=y, models=[model], savepath=os.getcwd(), plots=list())
        for d in splitter.splitdirs:
            self.assertTrue(os.path.exists(d))
            shutil.rmtree(d)
        return

    def test_close_comps(self):
        # Make entries at a 10% spacing
        composition_df = pd.DataFrame({'composition': ['Al{}Cu{}'.format(i, 10-i) for i in range(11)]})
        X = pd.DataFrame(np.random.uniform(low=0.0, high=100, size=(11, 10)))
        y = pd.Series(np.random.uniform(low=0.0, high=100, size=(11,)))

        # Generate test splits with a 5% distance cutoff
        splitter = LeaveCloseCompositionsOut(composition_df=composition_df, dist_threshold=0.5)

        model = SklearnModel(model='LinearRegression')
        splitter.evaluate(X=X, y=y, models=[model], savepath=os.getcwd(), plots=list())
        for d in splitter.splitdirs:
            self.assertTrue(os.path.exists(d))
            shutil.rmtree(d)
        return

    def test_leaveoutpercent(self):
        X = pd.DataFrame(np.random.uniform(low=0.0, high=100, size=(25, 10)))
        y = pd.Series(np.random.uniform(low=0.0, high=100, size=(25,)))
        splitter = LeaveOutPercent(percent_leave_out=0.20, n_repeats=5)
        model = SklearnModel(model='LinearRegression')
        splitter.evaluate(X=X, y=y, models=[model], groups=None, savepath=os.getcwd(), plots=list())
        for d in splitter.splitdirs:
            self.assertTrue(os.path.exists(d))
            shutil.rmtree(d)
        return

    def test_bootstrap(self):
        X = pd.DataFrame(np.random.uniform(low=0.0, high=100, size=(25, 10)))
        y = pd.Series(np.random.uniform(low=0.0, high=100, size=(25,)))
        splitter = Bootstrap(n=25, n_bootstraps=3, train_size=0.5)
        model = SklearnModel(model='LinearRegression')
        splitter.evaluate(X=X, y=y, models=[model], groups=None, savepath=os.getcwd(), plots=list())
        for d in splitter.splitdirs:
            self.assertTrue(os.path.exists(d))
            shutil.rmtree(d)
        return

    def test_justeachgroup(self):
        X = pd.DataFrame(np.random.uniform(low=0.0, high=100, size=(5, 10)))
        y = pd.Series(np.random.uniform(low=0.0, high=100, size=(5,)))
        groups = pd.DataFrame.from_dict({'groups': [0, 1, 1, 0, 1]})
        X = pd.concat([X, groups], axis=1)
        splitter = JustEachGroup()
        model = SklearnModel(model='LinearRegression')
        splitter.evaluate(X=X, y=y, models=[model], groups=X['groups'], savepath=os.getcwd(), plots=list())
        for d in splitter.splitdirs:
            self.assertTrue(os.path.exists(d))
            shutil.rmtree(d)
        return

    def test_leaveoutcluster(self):
        X = pd.DataFrame(np.random.uniform(low=0.0, high=100, size=(5, 10)))
        y = pd.Series(np.random.uniform(low=0.0, high=100, size=(5,)))
        splitter = LeaveOutClusterCV(cluster='KMeans', n_clusters=5)
        model = SklearnModel(model='LinearRegression')
        splitter.evaluate(X=X, y=y, models=[model], savepath=os.getcwd(), plots=list())
        for d in splitter.splitdirs:
            self.assertTrue(os.path.exists(d))
            shutil.rmtree(d)
        return

    def test_leaveouttwins(self):
        # SAME TEST AS OTHER SPLITTERS
        X = pd.DataFrame(np.random.uniform(low=0.0, high=100, size=(25, 10)))
        y = pd.Series(np.random.uniform(low=0.0, high=100, size=(25,)))
        splitter = LeaveOutTwinCV(threshold=0, auto_threshold=True)
        model = SklearnModel(model='LinearRegression')
        splitter.evaluate(X=X, y=y, models=[model], groups=None, savepath=os.getcwd(), plots=list())
        for d in splitter.splitdirs:
            self.assertTrue(os.path.exists(d))
            shutil.rmtree(d)

        # Setup to check exact twins
        splitter = LeaveOutTwinCV(threshold=0, auto_threshold=True)
        model = SklearnModel(model='LinearRegression')
        n_datapoints = 25
        n_features = 5

        # CASE 1: every datapoint is an exact twin, twins in both X and y
        X = pd.DataFrame(np.random.choice(range(-n_features*n_datapoints, n_features*n_datapoints), size=(n_datapoints, n_features), replace=False))  # This generates random numbers without repetitions
        # Pull out last row to be y
        y = X[n_features-1]
        X.drop(columns=n_features-1, inplace=True)
        # Create duplicates
        X = X.append(X.copy(), ignore_index=True)
        y = y.append(y.copy(), ignore_index=True)
        ret = splitter.split(X, y)
        numTwins = n_datapoints * 2
        for r in ret:
            self.assertTrue(len(r[0]) == numTwins or len(r[1]) == numTwins)  # check that everything was counted as a twin in each split

        # CASE 2: half datapoint is an exact twin, twins in both X and y
        X = pd.DataFrame(np.random.choice(range(-n_features*n_datapoints, n_features*n_datapoints), size=(n_datapoints, n_features), replace=False))
        y = X[n_features-1]
        X.drop(columns=n_features-1, inplace=True)
        X = X.append(X[0:int(n_datapoints/2)].copy(), ignore_index=True)
        y = y.append(y[0:int(n_datapoints/2)].copy(), ignore_index=True)
        ret = splitter.split(X, y)
        numTwins = int(n_datapoints/2)*2
        for r in ret:
            self.assertTrue(len(r[0]) == numTwins or len(r[1]) == numTwins)  # check correct number of twins

        # Setup to check twins generated by sci kit learn blobs
        splitter = LeaveOutTwinCV(threshold=2, debug=False, ceiling=1, auto_threshold=True)
        model = SklearnModel(model='LinearRegression')
        n_datapoints = 25
        n_features = 5

        cen = []
        for i in range(3):
            for j in range(3):
                cen.append((i*10, j*10))

        v = sk.make_blobs(n_samples=100, n_features=2, centers=cen, cluster_std=.1, shuffle=True, random_state=None)
        X = pd.DataFrame(v[0])
        y = pd.DataFrame(np.random.choice(range(-500, 500), size=(100, 1), replace=False))
        ret = splitter.split(X, y)
        numTwins = 100
        for r in ret:
            self.assertTrue(len(r[0]) == numTwins or len(r[1]) == numTwins)

        # plt.scatter(v[0][:, 0], v[0][:, 1], c=v[1])
        # plt.savefig("./test.png")
        # self.assertTrue(False, msg="In Progress")
        return


if __name__ == '__main__':
    unittest.main()
